using System.Net.Http.Headers;
using System.Net.Http.Json;
using System.Security.Cryptography.X509Certificates;
using FitConnect.Models;
using FitConnect.Security;
using Microsoft.Extensions.Logging;

namespace FitConnect.BaseClasses;

public abstract class FunctionalBaseClass: RestCallService {
    protected readonly ILogger? Logger;
    public readonly IEncryption Encryption;

    /// <summary>
    ///     Constructor for the FunctionalBaseClass
    /// </summary>
    /// <param name="logger">ILogger implementation</param>
    /// <param name="endpoints">FitConnect endpoints</param>
    /// <param name="certificate">The Encryption certificate</param>
    /// <example>
    /// new Sender(logger, FitConnectEndpoints.Create(FitConnectEndpoints.EndpointType.Development))
    /// </example>
    protected FunctionalBaseClass(ILogger? logger, FitConnectEndpoints? endpoints,
        X509Certificate2? certificate) {
        Endpoints = endpoints ??
                    FitConnectEndpoints.Create(FitConnectEndpoints.EndpointType.Development);

        Logger = logger;
        Encryption = new RsaEncryption(logger, certificate);
    }

    public FitConnectEndpoints Endpoints { get; }


    /// <summary>
    ///     Requesting an OAuth token from the FitConnect API.
    ///     <para>You can get the Client ID and Client Secret from the FitConnect Self Service portal
    ///     under <br/>
    ///     https://portal.auth-testing.fit-connect.fitko.dev</para>
    /// </summary>
    /// <param name="clientId">Your client Id</param>
    /// <param name="clientSecret">Your client Secret</param>
    /// <param name="scope">Scope if needed</param>
    /// <returns>The received token or null</returns>
    public async Task<OAuthAccessToken?> GetTokenAsync(string clientId, string clientSecret,
        string? scope = null) {
        var client = new HttpClient();
        client.DefaultRequestHeaders.Accept.Add(
            MediaTypeWithQualityHeaderValue.Parse("application/json"));

        var requestContent = new Dictionary<string, string> {
            { "grant_type", "client_credentials" },
            { "client_id", clientId },
            { "client_secret", clientSecret }
        };

        if (scope != null)
            requestContent["scope"] = scope;

        var content = new FormUrlEncodedContent(requestContent);

        var request = new HttpRequestMessage(HttpMethod.Post, Endpoints.TokenUrl) {
            Content = content,
            Method = HttpMethod.Post
        };

        var response = await client.SendAsync(request);

        return await response.Content.ReadFromJsonAsync<OAuthAccessToken>();
    }

    /// <summary>
    ///     Receive the SET data
    /// </summary>
    /// <returns></returns>
    public Task<SecurityEventToken> GetSetDataAsync() {
        throw new NotImplementedException();
    }


}